function feature = extractFeature(trainList, PARA)

% extractFeature(): train mixture of experts
% 
% model = extractFeature(trainList)
%     Input:
%       trainList: a list of training samples
%           - trainList.dir: directory of training image
%           - trainList.imageList{i}: ith training image
%     Output:
%       feature: 
%           feature.ceFeatureAll: color edge feature
%           feature.hdFeatureAll: high resolution depth feature
%           feature.ldFeatureAll: low resolution depth feature
% 
% Xue Tianfan, <xtf009@ie.cuhk.edu.hk>, MMLab CUHK

hblockSize = PARA.blockSize;
scale = PARA.scale;
lblockSize = hblockSize / scale;
maxImageSize = 1000;

ceFeatureAll = zeros(hblockSize*hblockSize, length(trainList.imageList) * maxImageSize^2 / hblockSize^2);
cesbFeatureAll = zeros(hblockSize*hblockSize, size(ceFeatureAll, 2));
hdFeatureAll = zeros(hblockSize*hblockSize, size(ceFeatureAll, 2));
ldFeatureAll = zeros(lblockSize*lblockSize, size(ceFeatureAll, 2));
lddFeatureAll = zeros(lblockSize*lblockSize*4, size(ceFeatureAll, 2));
ihddFeatureAll = zeros(hblockSize*hblockSize*4, size(ceFeatureAll, 2));
nbFeature = 0;

hf1 = [-1,0,1];
vf1 = [-1,0,1]';
hf2 = [1,0,-2,0,1];
vf2 = [1,0,-2,0,1]';

samplePerIm = floor(PARA.nsamples/length(trainList.imageList));

for i=1:length(trainList.imageList)
%    fprintf(1, 'deal image %s\n', trainList.imageList{i});
    
    % read image
    cim = rgb2gray(imread(sprintf('%s%s_c.bmp', trainList.dir, trainList.imageList{i})));
    ldim = imread(sprintf('%s%s_ld.bmp', trainList.dir, trainList.imageList{i}));
    hdim = imread(sprintf('%s%s_hd.bmp', trainList.dir, trainList.imageList{i}));
    ihdim = imresize(ldim, scale);
    ldimd1x = conv2(double(ldim), hf1,'same');
    ldimd1y = conv2(double(ldim), vf1,'same'); 
    ldimd2x = conv2(double(ldim), hf2,'same');
    ldimd2y = conv2(double(ldim), vf2,'same');
    ihdimd1x = conv2(double(ihdim), hf1,'same');
    ihdimd1y = conv2(double(ihdim), vf1,'same'); 
    ihdimd2x = conv2(double(ihdim), hf2,'same');
    ihdimd2y = conv2(double(ihdim), vf2,'same');
    hsize = size(hdim);
    lsize = size(ldim);
    if sum(lsize * scale == hsize) ~= 2
        throw(MException('VerifyOutput:OutOfBounds', 'Invalid image size'));
    end
%    [lCornerX, lCornerY] = meshgrid((0:1:lsize(2)-lblockSize), ....
%        (0:1:lsize(1)-lblockSize));

	if isfield(PARA, 'nsamples')
		[lCornerX, lCornerY] = meshgrid((0:1:lsize(2)-lblockSize),(0:1:lsize(1)-lblockSize));
		lCornerX = lCornerX(:)'; lCornerY = lCornerY(:)';
        selectIdx = randperm(size(lCornerX, 2));
        selectIdx = selectIdx(1:samplePerIm);
        lCornerX = lCornerX(selectIdx); lCornerY = lCornerY(selectIdx);
	else
		[lCornerX, lCornerY] = meshgrid((0:lblockSize:lsize(2)-lblockSize), ....
			(0:lblockSize:lsize(1)-lblockSize));
		lCornerX = lCornerX(:)';
		lCornerY = lCornerY(:)';
	end
    hCornerX = lCornerX * scale;
    hCornerY = lCornerY * scale;
    nbCorner = size(lCornerX, 2);
    
    % pre filter
%    ce = rgb2gray(cim);
%    ce = rgb2gray(imfilter(cim, fspecial('sobel')));
    ce = double(edge(cim, 'sobel', 0.01) .* ...
         imdilate(edge(imresize(ldim, scale), 'sobel', 0.01), strel('diamond', 1)));
    cesb = imfilter(double(cim), fspecial('sobel')) .* ...
         imdilate(edge(imresize(ldim, scale), 'sobel', 0.01), strel('diamond', 1)) / 255;
    
    % extract feature
    ceFeature = getPatch(double(ce), hCornerX, hCornerY, hblockSize, [0,0]);
    cesbFeature = getPatch(double(cesb), hCornerX, hCornerY, hblockSize, [0,0]);
    hdFeature = getPatch(double(hdim), hCornerX, hCornerY, hblockSize, [0,0]);
    ldFeature = getPatch(double(ldim), lCornerX, lCornerY, lblockSize, [0,0]);
    lddFeature = [getPatch(ldimd1x, lCornerX, lCornerY, lblockSize, [0,0]);...
        getPatch(ldimd1y, lCornerX, lCornerY, lblockSize, [0,0]);...
        getPatch(ldimd2x, lCornerX, lCornerY, lblockSize, [0,0]);...
        getPatch(ldimd2y, lCornerX, lCornerY, lblockSize, [0,0])];
    ihddFeature = [getPatch(ihdimd1x, hCornerX, hCornerY, hblockSize, [0,0]);...
        getPatch(ihdimd1y, hCornerX, hCornerY, hblockSize, [0,0]);...
        getPatch(ihdimd2x, hCornerX, hCornerY, hblockSize, [0,0]);...
        getPatch(ihdimd2y, hCornerX, hCornerY, hblockSize, [0,0])];
    
    % normalization
    % substract mean
    hdFeature = hdFeature - ones(hblockSize*hblockSize, 1) * mean(hdFeature);
    ldFeature = ldFeature - ones(lblockSize*lblockSize, 1) * mean(ldFeature);
    ceFeatureAll(:, nbFeature+1:nbFeature+nbCorner) = ceFeature/255;
    hdFeatureAll(:, nbFeature+1:nbFeature+nbCorner) = hdFeature/255;
    ldFeatureAll(:, nbFeature+1:nbFeature+nbCorner) = ldFeature/255;
    lddFeatureAll(:, nbFeature+1:nbFeature+nbCorner) = lddFeature;
    ihddFeatureAll(:, nbFeature+1:nbFeature+nbCorner) = ihddFeature;
    cesbFeatureAll(:, nbFeature+1:nbFeature+nbCorner) = cesbFeature;
    nbFeature = nbFeature + nbCorner;
end

feature.ceFeatureAll = ceFeatureAll(:, 1:nbFeature);
feature.cesbFeatureAll = cesbFeatureAll(:, 1:nbFeature);
feature.hdFeatureAll = hdFeatureAll(:, 1:nbFeature);
feature.ldFeatureAll = ldFeatureAll(:, 1:nbFeature);
feature.lddFeatureAll = lddFeatureAll(:, 1:nbFeature);
feature.ihddFeatureAll = ihddFeatureAll(:, 1:nbFeature);
% save('feature.mat', '-struct', 'feature');

end














